from IPython import display
from ensure import ensure_annotations
from urllib.request import urlopen
from myIPYNBrenderer.custom_exception import InvalidURLException
from myIPYNBrenderer.logger import logger


@ensure_annotations
def is_valid_URL(URL: str) -> bool:
    try:
        response_status = urlopen(URL).getcode()
        assert response_status == 200
        logger.debug(f"response_status: {response_status}")
        return True
    except Exception as e:
        logger.exception(e)
        return False


@ensure_annotations
def render_site(URL: str, width: str="90%", height: int=500) -> str:
    try:
        if is_valid_URL(URL):
            response = display.IFrame(src=URL, width=width, height=height)
            display.display(response)
            return "SUCCESS"
        else:
            raise InvalidURLException
    except Exception as e:
        raise e


import logging
import os
import sys

logging_str = "[%(asctime)s: %(levelname)s: %(module)s]: %(message)s"
log_dir ="./logs"
log_filepath = os.path.join(log_dir, "running_logs.log")
os.makedirs(log_dir, exist_ok=True)

logging.basicConfig(
    level= logging.INFO,
    format= logging_str,
    handlers= [
        logging.FileHandler(log_filepath),
        logging.StreamHandler(sys.stdout),
    ]
)

logger = logging.getLogger("myIPYNBrenderer")


from IPython import display
from ensure import ensure_annotations
from myIPYNBrenderer.custom_exception import InvalidURLException
from myIPYNBrenderer.logger import logger
from py_youtube import Data

ensure_annotations
def get_time_info(URL: str) -> int:

    def _verify_vid_id_len(vid_id, __expected_len=11):
        len_of_vid_id = len(vid_id)
        if len_of_vid_id != __expected_len:
            raise InvalidURLException(
                f"Invalid video id with length: {len_of_vid_id}, expected: {__expected_len}"
            )

    try:
        split_val = URL.split("=")
        if len(split_val) > 3:
            raise InvalidURLException
        if "watch" in URL:
            if "&t" in URL:
                vid_id, time = split_val[-2][:-2], int(split_val[-1][:-1])
                _verify_vid_id_len(vid_id)
                logger.info(f"video starts at: {time}")
                return time
            else:
                vid_id, time = split_val[-1], 0
                _verify_vid_id_len(vid_id)
                logger.info(f"video starts at: {time}")
                return time
        else:
            if "=" in URL and "?t" in URL:
                vid_id, time = split_val[0].split("/")[-1][:-2], int(split_val[-1])
                _verify_vid_id_len(vid_id)
                logger.info(f"video starts at: {time}")
                return time
            else:
                vid_id, time = URL.split("/")[-1], 0
                _verify_vid_id_len(vid_id)
                logger.info(f"video starts at: {time}")
                return time
    except Exception:
        raise InvalidURLException

@ensure_annotations
def render_YouTube_video(URL: str, width: int=900, height:int=500)-> str:
    try:
        if URL is None:
            raise InvalidURLException("URL cannot be None")
        data = Data(URL).data()
        
        if data["publishdate"] is not None:
            time = get_time_info(URL)
            vid_ID = data["id"]
            embed_URL = f"https://www.youtube.com/embed/{vid_ID}?start={time}"
            logger.info(f"embed URL: {embed_URL}")
            iframe = f"""<iframe
            width="{width}"
            height="{height}"
            src="{embed_URL}"
            title="YouTube video player"
            frameborder="0"
            allow="accelerometer;
            autoplay; clipboard-write;
            encrypted-media; gyroscope;
            picture-in-picture" allowfullscreen>
            </iframe>
            """
            display.display(display.HTML(iframe))
            return "SUCCESS"
        else:
            raise InvalidURLException
    except Exception as e:
        raise e


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as f:
    LONG_DESCRIPTION = f.read()

REPO_NAME = "myIPYNBrenderer"
AUTHOR_NAME = "Sumegh20" 
AUTHOR_EMAIL = "sumegh20@gmail.com"

setup(
    name = REPO_NAME,
    version = "0.0.5",
    author = AUTHOR_NAME,
    author_email = AUTHOR_EMAIL,
    description = "A small python package",
    long_description = LONG_DESCRIPTION,
    url = f"https://github.com/{AUTHOR_NAME}/{REPO_NAME}",
    project_urls = {
        "Bug Tracker": f"https://github.com/{AUTHOR_NAME}/{REPO_NAME}/issues"
    },
    package_dir = {"": "src"},
    packages = find_packages(where="src")
)
